@@ -15,6 +15,14 @@
 //#include <QContact>
 //#include <QContactManager>
 
+
+// wait for qt-mobility 1.1 + versit
+#include <QVersitOrganizerImporter>
+#include <QOrganizerItem>
+#include <QOrganizerManager>
+#include <QVersitReader>
+#include <QVersitDocument>
+
 #include "textwindow.h"
 
 #include <hildon-mime.h>
@@ -26,7 +34,7 @@
 #include <qurl.h>
 
 #include <QContactManager>
-using namespace QtMobility;
+
 #include <QContactPhoneNumber>
 #include <QContactName>
 
@@ -40,6 +48,9 @@ using namespace QtMobility;
 
 #include <QMaemo5InformationBox>
 
+
+using namespace QtMobility;
+
 //#define QT_NO_KEYWORDS
 //#include <libosso-abook/osso-abook-contact.h>
 //#include <libosso-abook/osso-abook-util.h>
@@ -80,7 +91,7 @@ void QrSink::clickAction(QWidget *parentWindow) {
         addMECARD(barcodeText);
 
     }else if(type==QrSink::Install){
-        //openInstallFile(barcodeText);
+        openInstallFile(barcodeText);
 
     }else if(type==QrSink::URL){
         // make sure our URL starts with http:// and ends at or before a space.
@@ -129,7 +140,8 @@ void QrSink::clickAction(QWidget *parentWindow) {
     }else if(type==QrSink::YouTube){
     }else if(type==QrSink::Ovi){
         openInstallFile(barcodeText);
-
+    }else if(type==QrSink::Apt){
+        openApt(barcodeText);
     }else if(type==QrSink::Unknown){
         // this is a catch all, as long as we are the last plugin that's tried that's fine
         // but wouldn't want to prevent some other plugin handling something we don't!
@@ -158,7 +170,7 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
             barcodeText=barcodeData;
             barcodeInformativeText = barcodeData; // todo, parse and add name of person here
 
-        }else if (barcodeData.indexOf("BEGIN:VCALENDAR",0,Qt::CaseInsensitive)==0){
+        }else if (barcodeData.indexOf("BEGIN:V",0,Qt::CaseInsensitive)==0){ // catch all payload types and pass to Versit
             type=this->QrSink::iCal;
             buttonText.append("Open iCal");
             barcodeText=barcodeData;
@@ -278,6 +290,16 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
             barcodeText=barcodeData;
             barcodeInformativeText = barcodeData;
 
+        }else if(barcodeData.indexOf("apt:",0,Qt::CaseInsensitive)==0) {
+            type=this->QrSink::Apt;
+            buttonText.append("Install package");
+            barcodeText=barcodeData;
+
+        }else if(barcodeData.indexOf("install:",0,Qt::CaseInsensitive)==0) {
+            type=this->QrSink::Install;
+            buttonText.append("Install package");
+            barcodeText=barcodeData;
+
         }else{
             type=this->QrSink::Unknown;
             buttonText.append("View text");
@@ -285,19 +307,295 @@ void QrSink::barcodeAnalysed(QString barcodeType, QString barcodeData) {
             barcodeInformativeText = barcodeData;
         }
 
-
+        qDebug() << "QRSink: " << "QR code reader is ready";
         emit isReady();
     }
 }
 
 
 
+/*
+ install:c:catalogue,names,with,comma,seps;p:package,names,with,comma,seps;d:
+
+
+
+ */
+
+void QrSink::openApt(QString data)
+{
+    if (data.indexOf("apt:") == 0) {
+        /**
+        * Is an APT URL - remove the scheme, which
+        * usually leaves us with the package name.
+        **/
+        data.remove(0, 4);
+
+        /* package name is in data - create a .install */
+
+        // ToDo - Make this handle the full range of options listed here
+        // https://wiki.ubuntu.com/AptUrl
+
+
+        QDir d("/home/user/.mbarcode/temp/");
+        if(!d.exists("/home/user/.mbarcode/temp/"))
+        d.mkpath("/home/user/.mbarcode/temp/");
+
+        //QFile installfile(QString("/home/user/.mbarcode/" + data + ".install"));
+        QTemporaryFile installfile("/home/user/.mbarcode/temp/aptXXXXXX.install");
+        if (installfile.open()) {
+
+            QString tempCatDef;
+            QTextStream tempCatDefStream(&tempCatDef);
+
+            QTextStream out(&installfile);
+            out << "[install]" << endl;
+
+            QStringList catalogueNames;
+
+            // split into sections separated by the & character
+            // fields are:
+            // packages=list of packages to be installed, semi-colon separated if required
+            // catalogues=extras|extras-testing|extras-devel|community-ssu (semi-colon separated)
+            // or install file info with fields comma separated i.e.
+            // name,uri,dist,components
+
+            QStringList items = data.split("&");
+            foreach(QString item, items){
+                if(item.indexOf("package")==0){
+                    // this is the list of packages
+
+                    // strip out any spaces to make sure our replace works correctly
+                    item.replace(" ","");
+
+                    // the install file requires package=, so remove the s if it's there
+                    item.replace("packages=", "package = ");
+
+                    out << item << endl; // write to the file
+
+                }else if(item.indexOf("catalog")==0){ // should be catalogues, but we need to match any possibilities for robustness
+
+                    // find the first =, this will be at the end of "catalogues =" (or whatever the user has actually written)
+                    int index = item.indexOf("=",0);
+
+                    // chop off the whole of the start section, we'll add it back ourselves later
+                    item.remove(0,index);
+
+                    // now split to see if we have multiple catalogues to add
+                    QStringList catalogueItems = data.split(";");
+                    foreach(QString catalogueItem, catalogueItems){
+                        if(catalogueItem.indexOf("community-testing")==0){
+                            catalogueNames << "community-testing";
+
+                            tempCatDefStream << "name = community-testing" << endl;
+                            tempCatDefStream << "uri = http://repository.maemo.org/community-testing" << endl;
+                            tempCatDefStream << "dist = fremantle" << endl;
+                            tempCatDefStream << "components = free non-free" << endl;
+                            tempCatDefStream << endl;
+
+                        }else if(catalogueItem.indexOf("extras-devel")==0){
+                            catalogueNames << "extras-devel";
+
+                            tempCatDefStream << "name = extras-devel" << endl;
+                            tempCatDefStream << "uri = http://repository.maemo.org/extras-devel" << endl;
+                            tempCatDefStream << "components = free non-free" << endl;
+                            tempCatDefStream << endl;
+
+                        }else if(catalogueItem.indexOf("extras-testing")==0){
+                            catalogueNames << "extras-testing";
+
+                            tempCatDefStream << "name = extras-testing" << endl;
+                            tempCatDefStream << "uri = http://repository.maemo.org/extras-testing" << endl;
+                            tempCatDefStream << "components = free non-free" << endl;
+                            tempCatDefStream << endl;
+
+                        }else if(catalogueItem.indexOf("extras")==0){
+                            catalogueNames << "extras";
+
+                            tempCatDefStream << "name = extras" << endl;
+                            tempCatDefStream << "uri = http://repository.maemo.org/extras" << endl;
+                            tempCatDefStream << "components = free non-free" << endl;
+                            tempCatDefStream << endl;
+
+                        }else{
+                            // this must be information to add a new catalogue
+                            // items will be comma separated
+                            QStringList catalogueElements = catalogueItem.split(",");
+
+                            // check the second element contains an url
+                            if(catalogueElements.at(1).indexOf("http://")!=0)
+                                continue;
+
+                            // there should be 4 elements: name, uri, dist, components
+                            if(catalogueElements.length()!=4)
+                                continue;
+
+                            // get rid of some whitespace if any present
+                            QString catName = catalogueElements.at(0).simplified();
+                            // remove other unwanted punctuation
+                            catName.replace(QRegExp("[^A-Za-z0-9_-]+"),"");
+
+
+                            catalogueNames << catName;
+                            tempCatDefStream << "name =" << catName << endl;
+                            tempCatDefStream << "uri = " << catalogueElements[1] << endl;
+                            if(catalogueElements.at(2).length()>0)
+                                tempCatDefStream << "dist = " << catalogueElements[2] << endl;
+                            tempCatDefStream << "components = " << catalogueElements[3] << endl;
+                            tempCatDefStream << endl;
+
+                        }
+                    }
+                }
+            }
+
+            if(catalogueNames.length()>0){
+                out << "catalogues = ";
+                for(int i=0; i<catalogueNames.length(); i++){
+                    out << catalogueNames.at(i);
+                    if(i!=catalogueNames.length()-1)
+                        out << "; ";
+                }
+                out << endl; // finish the line from above
+                out << endl; // add a blank line
+
+                // no need to flush a text stream pointing at a QString I take it...
+
+                out << tempCatDef; // add the actual catalogue definitions which we saved earlier
+            }
+            out.flush();
+
+            installfile.close();
+
+            openGeneric(QString("/home/user/.mbarcode/" + data + ".install"));
+
+            // this is done automatically
+            //installfile.remove(); // now remove the install file
+        }
+    }
+}
 
 
 // uses GLib, need to convert to pure Qt
 void QrSink::openInstallFile(QString data)
 {
-    Q_UNUSED(data)
+    if (data.indexOf("install:", Qt::CaseInsensitive) == 0) {
+
+        QDir d("/home/user/.mbarcode/temp/");
+        if(!d.exists("/home/user/.mbarcode/temp/"))
+            d.mkpath("/home/user/.mbarcode/temp/");
+
+        //QFile installfile(QString("/home/user/.mbarcode/" + data + ".install"));
+        QTemporaryFile installfile("/home/user/.mbarcode/temp/aptXXXXXX.install");
+        if (installfile.open()) {
+            QTextStream out(&installfile);
+            out << "[install]" << endl;
+
+            data.remove(0, 8); // remove the "install:" part
+
+            // split at semi-colons to generate a list of items for the file
+            QStringList listItem = data.split(";",QString::SkipEmptyParts);
+
+            foreach (const QString sItem, listItem) {
+                // see if this item is a package installation, or a catalogue installation or catalogue definition
+
+                if(sItem.indexOf("p:", Qt::CaseInsensitive)==0){
+                    // this is a package or list of packages
+                    out << "package = ";
+                    QStringList sElements = sItem.split(",", QString::SkipEmptyParts);
+                    foreach (const QString sElement, sElements) {
+                        out << sElement << "; ";
+                    }
+                    out << endl;
+
+                } else if (sItem.indexOf("c:", Qt::CaseInsensitive)==0){
+                    // this is a catalogue or list of catalogues
+                    out << "catalogues = ";
+                    QStringList sElements = sItem.split(",", QString::SkipEmptyParts);
+                    foreach (const QString sElement, sElements) {
+                        out << sElement << "; ";
+                    }
+                    out << endl;
+
+                } else if (sItem.indexOf("[", Qt::CaseInsensitive)==0){
+                    // this is the start of a catalogue definition
+                    QStringList sElements = sItem.split(",", QString::SkipEmptyParts);
+
+                    // see if we have any of the presets here
+                    if(sElements.length()==1){
+                        // yes we probably do
+                        if(sItem.indexOf("[extras]", Qt::CaseInsensitive)) {
+                            // add all the normal extras stuff in here
+
+                            out << "[extras]" << endl;
+                            out << "name = Maemo Extras" << endl;
+                            out << "uri = http://repository.maemo.org/extras/" << endl;
+                            out << "components = free non-free" << endl;
+
+
+                        } else if(sItem.indexOf("[extras-testing]", Qt::CaseInsensitive)) {
+                            // add all the normal extras-testing stuff in here
+
+                            out << "[extras-testing]" << endl;
+                            out << "name = Maemo Extras-testing" << endl;
+                            out << "uri = http://repository.maemo.org/extras-testing/" << endl;
+                            out << "components = free non-free" << endl;
+
+
+                        } else if(sItem.indexOf("[extras-devel]", Qt::CaseInsensitive)) {
+                            // add all the normal extras-devel stuff in here
+
+                            out << "[extras-devel]" << endl;
+                            out << "name = Maemo Extras-devel" << endl;
+                            out << "uri = http://repository.maemo.org/extras-devel/" << endl;
+                            out << "components = free non-free" << endl;
+
+                        } else if(sItem.indexOf("[community-testing]", Qt::CaseInsensitive)) {
+                            // add all the normal community-testing stuff in here
+
+                            out << "[community-testing]" << endl;
+                            out << "name = Maemo Community-testing" << endl;
+                            out << "uri = http://repository.maemo.org/community-testing/" << endl;
+                            out << "components = free non-free" << endl;
+
+                        }
+
+                    } else {
+                        // must be a custom repo
+
+                        foreach (QString sElement, sElements) {
+                            if(sElement.indexOf("[") && sElement.indexOf("]"))
+                                out << sElement << endl;
+                            else if (sElement.indexOf("n:")) // this is the repo name
+                                out << "name = " << sElement.remove(0,2) << endl;
+                            else if (sElement.indexOf("u:")) // this is the repo uri
+                                out << "uri = " << sElement.remove(0,2) << endl;
+                            else if (sElement.indexOf("c:")) // this is the repo components list
+                                out << "components = " << sElement.remove(0,2) << endl;
+                            else // no idea, so just add it in raw form :)
+                                out << sElement << endl;
+                        }
+                    }
+                }
+
+
+
+            }
+
+            out.flush();
+            installfile.close();
+            openGeneric(installfile.fileName());
+        }
+
+
+            /*/* XXX: Extras is hardcoded for now...
+            out << "catalogues = extras" << endl;
+            out << "package = " << data << endl;*/
+
+
+            // this is done automatically
+            //installfile.remove(); // now remove the install file
+
+    }
 //    FILE *install_file;
 //    gchar *data = g_strdup(data1);
 //    char *filename=NULL;
@@ -957,27 +1255,40 @@ void QrSink::addMECARD(QString user_data)
 
 void QrSink::addiCal(QString user_data)
 {
-    Q_UNUSED(user_data)
-    /*
-    // wait for qt-mobility 1.1 + versit
-    #include <QVersitOrganizerImporter>
-    #include <QOrganizerItem>
-    #include <QOrganizerManager>
+    //Q_UNUSED(user_data)
+
+//    // wait for qt-mobility 1.1 + versit
+//    #include <QVersitOrganizerImporter>
+//    #include <QOrganizerItem>
+//    #include <QOrganizerManager>
+
+    QOrganizerManager *manager = new QOrganizerManager();
 
     QVersitReader reader;
-    QBuffer b(user_data.toAscii(), this);
+    QByteArray ba(user_data.toAscii());
+    QBuffer b(&ba, this);
     b.open(QBuffer::ReadOnly);
     reader.setDevice(&b);
     reader.startReading();
     reader.waitForFinished();
-    QList<QVersitDocument> inputDocuments = reader.results();
 
     QVersitOrganizerImporter importer;
-    if (!importer.importDocuments(inputDocuments))
-         return;
-    QList<QOrganizerItem> items = importer.items();
-    QOrganizerManager::saveContacts(items);
-    */
+    foreach (const QVersitDocument& document, reader.results()) {
+        if (!importer.importDocument(document)) {
+            qWarning() << "Import failed, " << importer.errorMap();
+            continue;
+        }
+        QList<QOrganizerItem> items = importer.items();
+        QList<QOrganizerItem>::iterator it = items.begin();
+        while (it != items.end()) {
+            *it = manager->compatibleItem(*it);
+            it++;
+        }
+        manager->saveItems(&items);
+
+    }
+
+
 
 
 }
